#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const shelljs_1 = tslib_1.__importDefault(require("shelljs"));
const sade_1 = tslib_1.__importDefault(require("sade"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const execa_1 = tslib_1.__importDefault(require("execa"));
const path_1 = tslib_1.__importDefault(require("path"));
const ora_1 = tslib_1.__importDefault(require("ora"));
const fs_extra_1 = require("fs-extra");
const logError_1 = tslib_1.__importDefault(require("./logError"));
const messages_1 = require("./messages");
const utils_1 = require("./utils");
const getInstallCmd_1 = tslib_1.__importDefault(require("./getInstallCmd"));
const getInstallArgs_1 = tslib_1.__importDefault(require("./getInstallArgs"));
const enquirer_1 = require("enquirer");
const templates_1 = require("./templates");
const utils_2 = require("./templates/utils");
const config_1 = require("./config");
const package_json_1 = tslib_1.__importDefault(require("../package.json"));
const prog = sade_1.default('re-space');
const templateOptions = Object.keys(templates_1.templates);
prog
    .version(package_json_1.default.version)
    .command('create <pkg>')
    .describe('Create a new package')
    .example('create mypackage')
    .option('--template', `Specify a template. Allowed choices:
     
     [${templateOptions.join(', ')}]`)
    .example(`create --template ${templateOptions[0]} mypackage`)
    .action(async (pkg, opts) => {
    const respaceConfig = await config_1.getRespaceJson(opts);
    console.log(respaceConfig);
    console.log(chalk_1.default.magenta(`
    @re-space/cli
    `));
    const bootSpinner = ora_1.default(`Creating ${chalk_1.default.bold.green(pkg)}...`);
    let template;
    async function getProjectPath(projectPath) {
        const exists = await fs_extra_1.pathExists(projectPath);
        if (!exists) {
            return projectPath;
        }
        bootSpinner.fail(`Failed to create ${chalk_1.default.bold.red(pkg)}`);
        const prompt = new enquirer_1.Input({
            message: `A folder named ${chalk_1.default.bold.red(pkg)} already exists! ${chalk_1.default.bold('Choose a different name')}`,
            initial: pkg + '-copy',
            result: (v) => v.trim()
        });
        pkg = await prompt.run();
        projectPath = (await fs_extra_1.realpath(process.cwd())) + '/' + pkg;
        bootSpinner.start(`Creating ${chalk_1.default.bold.green(pkg)}...`);
        return await getProjectPath(projectPath);
    }
    try {
        const realPath = await fs_extra_1.realpath(process.cwd());
        const projectPath = await getProjectPath(`${realPath}/${respaceConfig.packages}/${pkg}`);
        console.log(projectPath);
        const prompt = new enquirer_1.Select({
            message: 'Choose a template',
            choices: templateOptions
        });
        if (opts.template) {
            template = opts.template.trim();
            if (!prompt.choices.includes(template)) {
                bootSpinner.fail(`Invalid template ${chalk_1.default.bold.red(template)}`);
                template = await prompt.run();
            }
        }
        else {
            template = await prompt.run();
        }
        bootSpinner.start();
        await fs_extra_1.copy(path_1.default.resolve(__dirname, `../../templates/${template}`), projectPath, {
            overwrite: true
        });
        // attempt to automatically derive author name
        let author = getAuthorName();
        if (!author) {
            bootSpinner.stop();
            const licenseInput = new enquirer_1.Input({
                name: 'author',
                message: 'Who is the package author?'
            });
            author = await licenseInput.run();
            setAuthorName(author);
            bootSpinner.start();
        }
        const templateConfig = templates_1.templates[template];
        const generatePackageJson = utils_2.composePackageJson(templateConfig);
        // Install deps
        process.chdir(projectPath);
        const safeName = utils_1.safePackageName(pkg);
        const pkgJson = generatePackageJson({ name: safeName, author });
        await fs_extra_1.outputJSON(path_1.default.resolve(projectPath, 'package.json'), pkgJson);
        bootSpinner.succeed(`Created ${chalk_1.default.bold.green(pkg)}`);
        await messages_1.start(pkg);
    }
    catch (error) {
        bootSpinner.fail(`Failed to create ${chalk_1.default.bold.red(pkg)}`);
        logError_1.default(error);
        process.exit(1);
    }
    const templateConfig = templates_1.templates[template];
    const { dependencies: deps } = templateConfig;
    const installSpinner = ora_1.default(messages_1.installing(deps.sort())).start();
    try {
        const cmd = await getInstallCmd_1.default();
        await execa_1.default(cmd, getInstallArgs_1.default(cmd, deps));
        installSpinner.succeed('Installed dependencies');
        console.log(await messages_1.start(pkg));
    }
    catch (error) {
        installSpinner.fail('Failed to install dependencies');
        logError_1.default(error);
        process.exit(1);
    }
});
function getAuthorName() {
    let author = '';
    author = shelljs_1.default
        .exec('npm config get init-author-name', { silent: true })
        .stdout.trim();
    if (author) {
        return author;
    }
    author = shelljs_1.default.exec('git config user.name', { silent: true }).stdout.trim();
    if (author) {
        setAuthorName(author);
        return author;
    }
    author = shelljs_1.default
        .exec('npm config get init-author-email', { silent: true })
        .stdout.trim();
    if (author) {
        return author;
    }
    author = shelljs_1.default.exec('git config user.email', { silent: true }).stdout.trim();
    if (author) {
        return author;
    }
    return author;
}
function setAuthorName(author) {
    shelljs_1.default.exec(`npm config set init-author-name "${author}"`, { silent: true });
}
prog.parse(process.argv);
